#!/usr/bin/python
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2010-2012 Red Hat, Inc.
#
# Authors:
# Thomas Woerner <twoerner@redhat.com>
# Jiri Popelka <jpopelka@redhat.com>

# To use in git tree: PYTHONPATH=.. python firewalld-test.py

import sys
import unittest
import firewall
from firewall.client import (
    FirewallClient,
    FirewallClientZoneSettings,
    FirewallClientServiceSettings,
    FirewallClientIcmpTypeSettings,
)
from firewall.core.base import DEFAULT_ZONE_TARGET


class TestFirewallDInterfaceConfig(unittest.TestCase):
    """
    For testing of permanent changes, ie. those that survive restart:
    """

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.fw = FirewallClient()

    def tearDown(self):
        unittest.TestCase.tearDown(self)

    def test_zones(self):
        """
        /org/fedoraproject/FirewallD1/config
            listZones()
            getZoneByName(String name)
            addZone(String name, Dict of {String, Variant} zone_settings)
        /org/fedoraproject/FirewallD1/config/zone/<id>
           getSettings()
           loadDefaults()
           update()
           rename()
           remove()
        """

        print("\nGetting invalid zone")
        self.assertRaisesRegex(
            Exception, "INVALID_ZONE", self.fw.config().getZoneByName, "dummyname"
        )

        zone_version = "1.0"
        zone_short = "Testing"
        zone_description = "this is just a testing zone"
        zone_target = DEFAULT_ZONE_TARGET
        zone_services = ["dhcpv6-client", "ssh"]
        zone_ports = [("123", "tcp"), ("666-667", "udp")]
        zone_icmpblocks = ["redirect", "echo-reply"]
        zone_masquerade = False
        zone_forward_ports = [
            ("443", "tcp", "441", "192.168.0.2"),
            ("123", "udp", "321", "192.168.1.1"),
        ]
        settings = FirewallClientZoneSettings()
        settings.setVersion(zone_version)
        settings.setShort(zone_short)
        settings.setDescription(zone_description)
        settings.setTarget(zone_target)
        settings.setServices(zone_services)
        settings.setPorts(zone_ports)
        settings.setIcmpBlocks(zone_icmpblocks)
        settings.setMasquerade(zone_masquerade)
        settings.setForwardPorts(zone_forward_ports)

        print("Adding zone with name that already exists")
        self.assertRaisesRegex(
            Exception, "NAME_CONFLICT", self.fw.config().addZone, "home", settings
        )
        print("Adding zone with empty name")
        self.assertRaisesRegex(
            Exception, "INVALID_NAME", self.fw.config().addZone, "", settings
        )
        zone_name = "test"
        print("Adding proper zone")
        self.fw.config().addZone(zone_name, settings)

        print("Checking the saved (permanent) settings")
        config_zone = self.fw.config().getZoneByName(zone_name)
        self.assertIsInstance(config_zone, firewall.client.FirewallClientConfigZone)
        zone_settings = config_zone.getSettings()
        self.assertIsInstance(zone_settings, firewall.client.FirewallClientZoneSettings)
        self.assertEqual(zone_settings.getVersion(), zone_version)
        self.assertEqual(zone_settings.getShort(), zone_short)
        self.assertEqual(zone_settings.getDescription(), zone_description)
        self.assertEqual(zone_settings.getTarget(), "default")
        self.assertEqual(zone_settings.getServices().sort(), zone_services.sort())
        self.assertEqual(zone_settings.getPorts().sort(), zone_ports.sort())
        self.assertEqual(zone_settings.getIcmpBlocks().sort(), zone_icmpblocks.sort())
        self.assertEqual(zone_settings.getMasquerade(), zone_masquerade)
        self.assertEqual(
            zone_settings.getForwardPorts().sort(), zone_forward_ports.sort()
        )

        print("Updating settings")
        zone_services.append("mdns")
        zone_settings.setServices(zone_services)
        config_zone.update(zone_settings)

        print("Reloading firewalld")
        self.fw.reload()

        print("Checking of runtime settings")
        self.assertTrue(zone_name in self.fw.getZones())
        self.assertEqual(self.fw.getServices(zone_name).sort(), zone_services.sort())
        self.assertEqual(self.fw.getPorts(zone_name).sort(), zone_ports.sort())
        self.assertEqual(
            self.fw.getIcmpBlocks(zone_name).sort(), zone_icmpblocks.sort()
        )
        self.assertEqual(self.fw.queryMasquerade(zone_name), zone_masquerade)
        self.assertEqual(
            self.fw.getForwardPorts(zone_name).sort(), zone_forward_ports.sort()
        )

        print("Checking that settings can be roundtripped through setZoneSettings")
        self.fw.setZoneSettings(zone_name, zone_settings)

        print("Renaming zone to name that already exists")
        config_zone = self.fw.config().getZoneByName(zone_name)
        self.assertRaisesRegex(Exception, "NAME_CONFLICT", config_zone.rename, "home")
        new_zone_name = "renamed"
        print("Renaming zone '%s' to '%s'" % (zone_name, new_zone_name))
        config_zone.rename(new_zone_name)

        print(
            "Checking whether the zone '%s' is accessible (it shouldn't be)" % zone_name
        )
        self.assertRaisesRegex(
            Exception, "INVALID_ZONE", self.fw.config().getZoneByName, zone_name
        )
        print("Checking whether the zone '%s' is accessible" % new_zone_name)
        config_zone = self.fw.config().getZoneByName(new_zone_name)
        zone_settings = config_zone.getSettings()
        self.assertEqual(zone_settings.getVersion(), zone_version)
        self.assertEqual(zone_settings.getShort(), zone_short)
        self.assertEqual(zone_settings.getDescription(), zone_description)
        self.assertEqual(zone_settings.getTarget(), "default")
        self.assertEqual(zone_settings.getServices().sort(), zone_services.sort())
        self.assertEqual(zone_settings.getPorts().sort(), zone_ports.sort())
        self.assertEqual(zone_settings.getIcmpBlocks().sort(), zone_icmpblocks.sort())
        self.assertEqual(zone_settings.getMasquerade(), zone_masquerade)
        self.assertEqual(
            zone_settings.getForwardPorts().sort(), zone_forward_ports.sort()
        )

        print("Removing the zone '%s'" % new_zone_name)
        config_zone.remove()
        print("Checking whether the removed zone is accessible (it shouldn't be)")
        self.assertRaisesRegex(
            Exception, "INVALID_ZONE", self.fw.config().getZoneByName, new_zone_name
        )

        # TODO test loadDefaults() ?

    def test_services(self):
        """
        /org/fedoraproject/FirewallD1/config
            listServices()
            getServiceByName(String name)
            addService(String name, Dict of {String, Variant} settings)

        /org/fedoraproject/FirewallD1/config/service/<id>
           getSettings()
           loadDefaults()
           update()
           rename()
           remove()
        """

        print("\nGetting invalid service")
        self.assertRaisesRegex(
            Exception, "INVALID_SERVICE", self.fw.config().getServiceByName, "dummyname"
        )

        service_version = "1.0"
        service_short = "Testing"
        service_description = "this is just a testing service"
        service_ports = [("123", "tcp"), ("666-667", "udp")]
        service_modules = ["nf_conntrack_tftp"]
        service_destinations = {"ipv4": "1.2.3.4", "ipv6": "dead::beef"}
        settings = FirewallClientServiceSettings()  # ["", "", "", [], [], {}]
        settings.setVersion(service_version)
        settings.setShort(service_short)
        settings.setDescription(service_description)
        settings.setPorts(service_ports)
        settings.setModules(service_modules)
        settings.setDestinations(service_destinations)

        print("Adding service with name that already exists")
        self.assertRaisesRegex(
            Exception, "NAME_CONFLICT", self.fw.config().addService, "mdns", settings
        )
        print("Adding service with empty name")
        self.assertRaisesRegex(
            Exception, "INVALID_NAME", self.fw.config().addService, "", settings
        )
        service_name = "test"
        print("Adding proper service")
        self.fw.config().addService(service_name, settings)

        print("Checking the saved (permanent) settings")
        config_service = self.fw.config().getServiceByName(service_name)
        self.assertIsInstance(
            config_service, firewall.client.FirewallClientConfigService
        )
        service_settings = config_service.getSettings()
        self.assertIsInstance(
            service_settings, firewall.client.FirewallClientServiceSettings
        )

        print("Updating settings")
        service_modules.append("nf_conntrack_sip")
        service_destinations["ipv6"] = "3ffe:501:ffff::"
        service_settings.setModules(service_modules)
        service_settings.setDestinations(service_destinations)
        config_service.update(service_settings)
        self.assertEqual(service_settings.getVersion(), service_version)
        self.assertEqual(service_settings.getShort(), service_short)
        self.assertEqual(service_settings.getDescription(), service_description)
        self.assertEqual(service_settings.getPorts().sort(), service_ports.sort())
        self.assertEqual(service_settings.getModules().sort(), service_modules.sort())
        self.assertDictEqual(service_settings.getDestinations(), service_destinations)

        print("Renaming service to name that already exists")
        config_service = self.fw.config().getServiceByName(service_name)
        self.assertRaisesRegex(
            Exception, "NAME_CONFLICT", config_service.rename, "mdns"
        )
        new_service_name = "renamed"
        print("Renaming service '%s' to '%s'" % (service_name, new_service_name))
        config_service.rename(new_service_name)

        print(
            "Checking whether the service '%s' is accessible (it shouldn't be)"
            % service_name
        )
        self.assertRaisesRegex(
            Exception,
            "INVALID_SERVICE",
            self.fw.config().getServiceByName,
            service_name,
        )
        print("Checking whether the service '%s' is accessible" % new_service_name)
        config_service = self.fw.config().getServiceByName(new_service_name)
        service_settings = config_service.getSettings()
        self.assertEqual(service_settings.getVersion(), service_version)
        self.assertEqual(service_settings.getShort(), service_short)
        self.assertEqual(service_settings.getDescription(), service_description)
        self.assertEqual(service_settings.getPorts().sort(), service_ports.sort())
        self.assertEqual(service_settings.getModules().sort(), service_modules.sort())
        self.assertDictEqual(service_settings.getDestinations(), service_destinations)

        print("Removing the service '%s'" % new_service_name)
        config_service.remove()
        print("Checking whether the removed service is accessible (it shouldn't be)")
        self.assertRaisesRegex(
            Exception,
            "INVALID_SERVICE",
            self.fw.config().getServiceByName,
            new_service_name,
        )

        # TODO test loadDefaults() ?

    def test_icmptypes(self):
        """
        /org/fedoraproject/FirewallD1/config
            listIcmpTypes()
            getIcmpTypeByName(String name)
            addIcmpType(String name, Dict of {String, Variant} settings)

        /org/fedoraproject/FirewallD1/config/icmptype/<id>
           getSettings()
           loadDefaults()
           update()
           rename()
           remove()
        """
        print("\nGetting invalid icmp-type")
        self.assertRaisesRegex(
            Exception,
            "INVALID_ICMPTYPE",
            self.fw.config().getIcmpTypeByName,
            "dummyname",
        )

        icmptype_version = "1.0"
        icmptype_short = "Testing"
        icmptype_description = "this is just a testing icmp type"
        icmptype_destinations = ["ipv4"]
        settings = FirewallClientIcmpTypeSettings()  # ["", "", "", []]
        settings.setVersion(icmptype_version)
        settings.setShort(icmptype_short)
        settings.setDescription(icmptype_description)
        settings.setDestinations(icmptype_destinations)

        print("Adding icmp type with name that already exists")
        self.assertRaisesRegex(
            Exception,
            "NAME_CONFLICT",
            self.fw.config().addIcmpType,
            "echo-reply",
            settings,
        )
        print("Adding icmp type with empty name")
        self.assertRaisesRegex(
            Exception, "INVALID_NAME", self.fw.config().addIcmpType, "", settings
        )
        icmptype_name = "test"
        print("Adding proper icmp type")
        self.fw.config().addIcmpType(icmptype_name, settings)

        print("Checking the saved (permanent) settings")
        config_icmptype = self.fw.config().getIcmpTypeByName(icmptype_name)
        self.assertIsInstance(
            config_icmptype, firewall.client.FirewallClientConfigIcmpType
        )
        icmptype_settings = config_icmptype.getSettings()
        self.assertIsInstance(
            icmptype_settings, firewall.client.FirewallClientIcmpTypeSettings
        )

        print("Updating settings")
        icmptype_destinations.append("ipv6")
        icmptype_settings.setDestinations(icmptype_destinations)
        config_icmptype.update(icmptype_settings)
        self.assertEqual(icmptype_settings.getVersion(), icmptype_version)
        self.assertEqual(icmptype_settings.getShort(), icmptype_short)
        self.assertEqual(icmptype_settings.getDescription(), icmptype_description)
        self.assertEqual(
            icmptype_settings.getDestinations().sort(), icmptype_destinations.sort()
        )

        print("Renaming icmp type to name that already exists")
        config_icmptype = self.fw.config().getIcmpTypeByName(icmptype_name)
        self.assertRaisesRegex(
            Exception, "NAME_CONFLICT", config_icmptype.rename, "echo-reply"
        )
        new_icmptype_name = "renamed"
        print("Renaming icmp type '%s' to '%s'" % (icmptype_name, new_icmptype_name))
        config_icmptype.rename(new_icmptype_name)

        print(
            "Checking whether the icmp type '%s' is accessible (it shouldn't be)"
            % icmptype_name
        )
        self.assertRaisesRegex(
            Exception,
            "INVALID_ICMPTYPE",
            self.fw.config().getIcmpTypeByName,
            icmptype_name,
        )
        print("Checking whether the icmp type '%s' is accessible" % new_icmptype_name)
        config_icmptype = self.fw.config().getIcmpTypeByName(new_icmptype_name)
        icmptype_settings = config_icmptype.getSettings()
        self.assertEqual(icmptype_settings.getVersion(), icmptype_version)
        self.assertEqual(icmptype_settings.getShort(), icmptype_short)
        self.assertEqual(icmptype_settings.getDescription(), icmptype_description)
        self.assertEqual(
            icmptype_settings.getDestinations().sort(), icmptype_destinations.sort()
        )

        print("Removing the icmp type '%s'" % new_icmptype_name)
        config_icmptype.remove()
        print("Checking whether the removed icmp type is accessible (it shouldn't be)")
        self.assertRaisesRegex(
            Exception,
            "INVALID_ICMPTYPE",
            self.fw.config().getIcmpTypeByName,
            new_icmptype_name,
        )

        # TODO test loadDefaults() ?


if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestFirewallDInterfaceConfig)
    results = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(0 if results.wasSuccessful() else 1)



#!/usr/bin/python
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2010-2012 Red Hat, Inc.
#
# Authors:
# Thomas Woerner <twoerner@redhat.com>
# Jiri Popelka <jpopelka@redhat.com>

# To use in git tree: PYTHONPATH=.. python firewalld-test.py

import dbus
import sys
import time
import unittest

from firewall.config.dbus import (
    DBUS_PATH,
    DBUS_PATH_CONFIG,
    DBUS_INTERFACE,
    DBUS_INTERFACE_ZONE,
    DBUS_INTERFACE_CONFIG,
)
from firewall.dbus_utils import dbus_to_python
from pprint import pprint


class TestFirewallD(unittest.TestCase):
    """
    For testing of temporary changes, ie. those that disappear with restart:
    adding/removing interfaces to zones, setting/changing of default zone
    adding/removing of services, ports, forward ports, icmp blocks
    """

    def setUp(self):
        unittest.TestCase.setUp(self)
        bus = dbus.SystemBus()
        dbus_obj = bus.get_object(DBUS_INTERFACE, DBUS_PATH)
        dbus_obj_config = bus.get_object(DBUS_INTERFACE, DBUS_PATH_CONFIG)
        self.fw = dbus.Interface(dbus_obj, dbus_interface=DBUS_INTERFACE)
        self.fw_zone = dbus.Interface(dbus_obj, dbus_interface=DBUS_INTERFACE_ZONE)
        self.config_properties = dbus.Interface(
            dbus_obj_config, dbus_interface="org.freedesktop.DBus.Properties"
        )
        self.config_properties.Set(DBUS_INTERFACE_CONFIG, "FlushAllOnReload", "no")
        self.fw.reload()

    def test_zone_getActiveZones(self):
        interface = "baz"
        zone = "home"

        print("\nAdding interface '%s' to '%s' zone" % (interface, zone))
        self.fw_zone.addInterface(zone, interface)

        print("Getting active zones: ")
        ret = self.fw_zone.getActiveZones()
        self.assertTrue(len(ret) > 0)
        pprint(dbus_to_python(ret))

        self.fw_zone.removeInterface(zone, interface)  # cleanup

    def test_zone_getZones(self):
        z = self.fw_zone.getZones()
        print("\nZones:")
        pprint(dbus_to_python(z))

    def test_zone_add_remove_queryInterface(self):
        interface = "foo"
        zone = "trusted"

        print("\nAdding interface '%s' to '%s' zone" % (interface, zone))
        ret = self.fw_zone.addInterface(zone, interface)
        self.assertEqual(ret, zone)
        self.assertTrue(self.fw_zone.queryInterface(zone, interface))

        print("Re-adding")
        self.assertRaisesRegex(
            Exception, "ZONE_ALREADY_SET", self.fw_zone.addInterface, zone, interface
        )

        zone = "block"
        print("Re-adding interface '%s' to '%s' zone" % (interface, zone))
        self.assertRaisesRegex(
            Exception, "ZONE_CONFLICT", self.fw_zone.addInterface, zone, interface
        )

        print("Removing interface '%s' from '%s' zone" % (interface, zone))
        self.assertRaisesRegex(
            Exception, "ZONE_CONFLICT", self.fw_zone.removeInterface, zone, interface
        )

        zone = "trusted"
        print("Removing interface '%s' from '%s' zone" % (interface, zone))
        ret = self.fw_zone.removeInterface(zone, interface)
        self.assertEqual(ret, zone)
        self.assertFalse(self.fw_zone.queryInterface(zone, interface))
        print("Re-removing")
        self.assertRaises(Exception, self.fw_zone.removeInterface, zone, interface)

        print("Add again and remove interface '%s' from zone it belongs to" % interface)
        self.fw_zone.addInterface(zone, interface)
        self.assertTrue(self.fw_zone.queryInterface(zone, interface))
        ret = self.fw_zone.removeInterface("", interface)
        self.assertEqual(ret, zone)
        self.assertFalse(self.fw_zone.queryInterface(zone, interface))
        print("Re-removing")
        self.assertRaises(Exception, self.fw_zone.removeInterface, "", interface)

    def test_zone_change_queryZone(self):
        interface = "foo"
        zone = "internal"

        print("\nChanging zone of interface '%s' to '%s'" % (interface, zone))
        ret = self.fw_zone.changeZone(zone, interface)
        self.assertEqual(ret, zone)
        self.assertTrue(self.fw_zone.queryInterface(zone, interface))

        print("Get zone of interface '%s': " % (interface))
        ret = self.fw_zone.getZoneOfInterface(interface)
        self.assertEqual(ret, zone)
        print(dbus_to_python(ret))

        self.fw_zone.removeInterface(zone, interface)  # cleanup

    def test_zone_add_get_query_removeService(self):
        service = "samba"
        zone = "external"
        print("\nAdding service '%s' to '%s' zone" % (service, zone))
        ret = self.fw_zone.addService(zone, service, 0)
        self.assertEqual(ret, zone)
        print("Re-adding")
        self.assertRaisesRegex(
            Exception, "ALREADY_ENABLED", self.fw_zone.addService, zone, service, 0
        )

        print("Get services of zone '%s'" % (zone))
        ret = self.fw_zone.getServices(zone)
        self.assertTrue(len(ret) > 0)
        pprint(dbus_to_python(ret))

        print("Removing service '%s' from '%s' zone" % (service, zone))
        ret = self.fw_zone.removeService(zone, service)
        self.assertEqual(ret, zone)
        print("Re-removing")
        self.assertRaisesRegex(
            Exception, "NOT_ENABLED", self.fw_zone.removeService, zone, service
        )

        zone = "dmz"
        timeout = 2
        print(
            "Adding timed service '%s' to '%s' zone, active for %d seconds"
            % (service, zone, timeout)
        )
        ret = self.fw_zone.addService(zone, service, timeout)
        self.assertEqual(ret, zone)
        self.assertTrue(self.fw_zone.queryService(zone, service))
        time.sleep(timeout + 1)
        print("Checking if timeout has been working")
        self.assertFalse(self.fw_zone.queryService(zone, service))

    def test_zone_add_get_query_removePort(self):
        port = "443"
        protocol = "tcp"
        zone = "public"
        print("\nAdding port '%s/%s' to '%s' zone" % (port, protocol, zone))
        ret = self.fw_zone.addPort(zone, port, protocol, 0)
        self.assertEqual(ret, zone)
        print("Re-adding port")
        self.assertRaisesRegex(
            Exception, "ALREADY_ENABLED", self.fw_zone.addPort, zone, port, protocol, 0
        )

        print("Get ports of zone '%s': " % (zone))
        ret = self.fw_zone.getPorts(zone)
        self.assertTrue(len(ret) > 0)
        pprint(dbus_to_python(ret))

        print("Removing port '%s/%s' from '%s' zone" % (port, protocol, zone))
        ret = self.fw_zone.removePort(zone, port, protocol)
        self.assertEqual(ret, zone)
        print("Re-removing")
        self.assertRaisesRegex(
            Exception, "NOT_ENABLED", self.fw_zone.removePort, zone, port, protocol
        )

        port = "443-445"
        protocol = "udp"
        zone = "dmz"
        timeout = 2
        print(
            "Adding timed port '%s/%s' to '%s' zone, active for %d seconds"
            % (port, protocol, zone, timeout)
        )
        ret = self.fw_zone.addPort(zone, port, protocol, timeout)
        self.assertEqual(ret, zone)
        self.assertTrue(self.fw_zone.queryPort(zone, port, protocol))
        time.sleep(timeout + 1)
        print("Checking if timeout has been working")
        self.assertFalse(self.fw_zone.queryPort(zone, port, protocol))

    def test_zone_add_query_removeMasquerade(self):
        zone = "public"
        print("\nAdd masquerade to '%s' zone" % (zone))
        ret = self.fw_zone.addMasquerade(zone, 0)
        self.assertEqual(ret, zone)
        print("Re-adding")
        self.assertRaisesRegex(
            Exception, "ALREADY_ENABLED", self.fw_zone.addMasquerade, zone, 0
        )

        print("Checking if masquerade is added to zone '%s'" % (zone))
        self.assertTrue(self.fw_zone.queryMasquerade(zone))

        print("Remove masquerade from '%s' zone" % (zone))
        ret = self.fw_zone.removeMasquerade(zone)
        self.assertEqual(ret, zone)
        print("Re-adding")
        self.assertRaisesRegex(
            Exception, "NOT_ENABLED", self.fw_zone.removeMasquerade, zone
        )

        zone = "dmz"
        timeout = 2
        print(
            "Add timed masquerade to '%s' zone, active for %d seconds" % (zone, timeout)
        )
        ret = self.fw_zone.addMasquerade(zone, timeout)
        self.assertEqual(ret, zone)
        self.assertTrue(self.fw_zone.queryMasquerade(zone))
        time.sleep(timeout + 1)
        print("Checking if timeout has been working")
        self.assertFalse(self.fw_zone.queryMasquerade(zone))

    def test_zone_add_get_query_removeForwardPort(self):
        port = "443"
        protocol = "tcp"
        toport = "441"
        toaddr = "192.168.0.2"
        zone = "public"
        print(
            "\nAdding forward port '%s/%s' to '%s:%s' to '%s' zone"
            % (port, protocol, toaddr, toport, zone)
        )
        ret = self.fw_zone.addForwardPort(zone, port, protocol, toport, toaddr, 0)
        self.assertEqual(ret, zone)
        print("Re-adding")
        self.assertRaisesRegex(
            Exception,
            "ALREADY_ENABLED",
            self.fw_zone.addForwardPort,
            zone,
            port,
            protocol,
            toport,
            toaddr,
            0,
        )

        print("Get forward ports of zone '%s': " % (zone))
        ret = self.fw_zone.getForwardPorts(zone)
        self.assertTrue(len(ret) > 0)
        pprint(dbus_to_python(ret))

        print(
            "Removing forward port '%s/%s' to '%s:%s' from '%s' zone"
            % (port, protocol, toaddr, toport, zone)
        )
        ret = self.fw_zone.removeForwardPort(zone, port, protocol, toport, toaddr)
        self.assertEqual(ret, zone)
        print("Re-removing")
        self.assertRaisesRegex(
            Exception,
            "NOT_ENABLED",
            self.fw_zone.removeForwardPort,
            zone,
            port,
            protocol,
            toport,
            toaddr,
        )

        port = "443-445"
        protocol = "udp"
        toport = ""
        toaddr = "192.168.0.3"
        zone = "dmz"
        timeout = 2
        print(
            "Adding timed forward port '%s/%s' to '%s:%s' to '%s' zone, active for %d seconds"
            % (port, protocol, toaddr, toport, zone, timeout)
        )
        ret = self.fw_zone.addForwardPort(zone, port, protocol, toport, toaddr, timeout)
        self.assertEqual(ret, zone)
        self.assertTrue(
            self.fw_zone.queryForwardPort(zone, port, protocol, toport, toaddr)
        )
        time.sleep(timeout + 1)
        print("Checking if timeout has been working")
        self.assertFalse(
            self.fw_zone.queryForwardPort(zone, port, protocol, toport, toaddr)
        )

    def test_zone_add_get_query_removeIcmpBlock(self):
        icmp = "parameter-problem"
        zone = "external"
        print("\nAdding icmp block '%s' to '%s' zone" % (icmp, zone))
        ret = self.fw_zone.addIcmpBlock(zone, icmp, 0)
        self.assertEqual(ret, zone)
        print("Re-adding")
        self.assertRaisesRegex(
            Exception, "ALREADY_ENABLED", self.fw_zone.addIcmpBlock, zone, icmp, 0
        )

        print("Get icmp blocks of zone '%s': " % (zone))
        ret = self.fw_zone.getIcmpBlocks(zone)
        self.assertTrue(len(ret) > 0)
        pprint(dbus_to_python(ret))

        print("Removing icmp block '%s' from '%s' zone" % (icmp, zone))
        ret = self.fw_zone.removeIcmpBlock(zone, icmp)
        self.assertEqual(ret, zone)
        print("Re-removing")
        self.assertRaisesRegex(
            Exception, "NOT_ENABLED", self.fw_zone.removeIcmpBlock, zone, icmp
        )

        icmp = "redirect"
        zone = "dmz"
        timeout = 2
        print(
            "Adding timed icmp block '%s' to '%s' zone, active for %d seconds: "
            % (icmp, zone, timeout)
        )
        ret = self.fw_zone.addIcmpBlock(zone, icmp, timeout)
        self.assertEqual(ret, zone)
        self.assertTrue(self.fw_zone.queryIcmpBlock(zone, icmp))
        time.sleep(timeout + 1)
        print("Checking if timeout has been working: ")
        self.assertFalse(self.fw_zone.queryIcmpBlock(zone, icmp))

    def test_reload(self):
        interface = "foo"
        zone = "work"

        self.fw_zone.addInterface(zone, interface)
        self.fw.reload()
        print("\nChecking if interface remains in zone after service reload: ")
        self.assertTrue(self.fw_zone.queryInterface(zone, interface))

        self.fw_zone.removeInterface(zone, interface)  # cleanup


if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestFirewallD)
    results = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(0 if results.wasSuccessful() else 1)
